package com.example.demo;

import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import com.example.demo.items.model.ItemEntity;
import com.example.demo.items.service.ItemService;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

@SpringBootApplication
public class DemoApplication implements CommandLineRunner {
	// Логгер
	private final Logger log = LoggerFactory.getLogger(DemoApplication.class);

	// Бизнес-логика для сущности "Тип" (Тип книг)
	private final TypeService typeService;

	// Бизнес-логика для сущности "Заказ" (Заказ, содержащий книги)
	private final ItemService itemService;

	// Бизнес-логика для сущности "Пользователь"
	private final UserService userService;

	// Бизнес-логика для сущности "Сообщение"
	private final MessageService messageService;

	// Конструктор
	public DemoApplication(TypeService typeService, ItemService itemService, UserService userService, MessageService messageService) {
		this.typeService = typeService;
		this.itemService = itemService;
		this.userService = userService;
		this.messageService = messageService;
	}

	// Входная точка программы
	public static void main(String[] args) {
		SpringApplication.run(DemoApplication.class, args);
	}

	@Override
	public void run(String... args) throws Exception {
		if (args.length > 0 && Objects.equals("--populate", args[0])) {
			log.info("Create default types values");
			final var type1 = typeService.create(new TypeEntity(null, "Protection"));
			final var type2 = typeService.create(new TypeEntity(null, "Sharpness"));
			final var type3 = typeService.create(new TypeEntity(null, "Infinity"));

			log.info("Create default items values");
			itemService.create(new ItemEntity(null, type1, 50.00, 20));
			itemService.create(new ItemEntity(null, type1, 12.00, 3));
			itemService.create(new ItemEntity(null, type2, 15.00, 30));
			itemService.create(new ItemEntity(null, type2, 64.00, 10));
			itemService.create(new ItemEntity(null, type2, 15.00, 6));
			itemService.create(new ItemEntity(null, type3, 80.00, 6));
			itemService.create(new ItemEntity(null, type3, 64.00, 3));

			log.info("Create default users values");
			final var user1 = userService.create(new UserEntity(null, "User1", "password1", "mail1@gmail.com"));
			final var user2 = userService.create(new UserEntity(null, "User2", "password2", "mail2@gmail.com"));
			final var user3 = userService.create(new UserEntity(null, "User3", "password3", "mail3@gmail.com"));

			log.info("Create default messages values");
			messageService.create(new MessageEntity(null, user1, "Message1"));
			messageService.create(new MessageEntity(null, user1, "Message2"));
			messageService.create(new MessageEntity(null, user2, "Message3"));
			messageService.create(new MessageEntity(null, user2, "Message4"));
			messageService.create(new MessageEntity(null, user2, "Message5"));
			messageService.create(new MessageEntity(null, user3, "Message6"));
			messageService.create(new MessageEntity(null, user3, "Message7"));
		}
	}
}
