package com.example.demo.items.service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.items.model.ItemEntity;
import com.example.demo.items.repository.ItemRepository;

// Бизнес-логика для сущности "Заказ" (Заказ, содержащий книги)
@Service
public class ItemService {
    // Хранилище элементов
    private final ItemRepository repository;

    // Конструктор
    public ItemService(ItemRepository repository) {
        this.repository = repository;
    }

    // Получить все элементы или по заданному фильтру
    public List<ItemEntity> getAll(Long typeId) {
        if (Objects.equals(typeId, 0L)) {
            return repository.getAll();
        }
        return repository.getAll().stream()
                .filter(item -> item.getType().getId().equals(typeId))
                .toList();
    }

    // Получить элемент по идентификатору
    public ItemEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    // Создать элемент
    public ItemEntity create(ItemEntity entity) {
        return repository.create(entity);
    }

    // Изменить элемент
    public ItemEntity update(Long id, ItemEntity entity) {
        final ItemEntity existsEntity = get(id);
        existsEntity.setType(entity.getType());
        existsEntity.setPrice(entity.getPrice());
        existsEntity.setCount(entity.getCount());
        return repository.update(existsEntity);
    }

    // Удалить элемент
    public ItemEntity delete(Long id) {
        final ItemEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
