package com.example.demo.messages.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.core.configuration.Constants;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

// Контроллер для сущности "Сообщение"
@RestController
@RequestMapping(Constants.API_URL + "/message")
public class MessageController {
    // Бизнес-логика для сущности "Сообщение"
    private final MessageService messageService;

    // Бизнес-логика для сущности "Пользователь"
    private final UserService userService;

    // Библиотека для преобразования сущности
    private final ModelMapper modelMapper;

    // Конструктор
    public MessageController(MessageService messageService, UserService userService, ModelMapper modelMapper) {
        this.messageService = messageService;
        this.userService = userService;
        this.modelMapper = modelMapper;
    }

    // Преобразовать из сущности в DTO
    private MessageDto toDto(MessageEntity entity) {
        return modelMapper.map(entity, MessageDto.class);
    }

    // Преобразовать из DTO в сущность
    private MessageEntity toEntity(@Valid MessageDto dto) {
        final MessageEntity entity = modelMapper.map(dto, MessageEntity.class);
        entity.setSender(userService.get(dto.getSenderId()));
        return entity;
    }

    // Получить все элементы
    @GetMapping
    public List<MessageDto> getAll(@RequestParam(name = "senderId", defaultValue = "0") Long senderId) {
        return messageService.getAll(senderId).stream().map(this::toDto).toList();
    }

    // Получить элемент по идентификатору
    @GetMapping("/{id}")
    public MessageDto get(@PathVariable(name = "id") Long id) {
        return toDto(messageService.get(id));
    }

    // Создать элемент
    @PostMapping
    public MessageDto create(@RequestBody @Valid MessageDto dto) {
        return toDto(messageService.create(toEntity(dto)));
    }

    // Изменить элемент
    @PutMapping("/{id}")
    public MessageDto update(@PathVariable(name = "id") Long id, @RequestBody @Valid MessageDto dto) {
        return toDto(messageService.update(id, toEntity(dto)));
    }

    // Удалить элемент
    @DeleteMapping("/{id}")
    public MessageDto delete(@PathVariable(name = "id") Long id) {
        return toDto(messageService.delete(id));
    }
}
