package com.example.demo.messages.service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.repository.MessageRepository;

// Бизнес-логика для сущности "Сообщение"
@Service
public class MessageService {
    // Хранилище элементов
    private final MessageRepository repository;

    // Конструктор
    public MessageService(MessageRepository repository) {
        this.repository = repository;
    }

    // Получить все элементы или по заданному фильтру
    public List<MessageEntity> getAll(Long senderId) {
        if (Objects.equals(senderId, 0L)) {
            return repository.getAll();
        }
        return repository.getAll().stream()
                .filter(item -> item.getSender().getId().equals(senderId))
                .toList();
    }

    // Получить элемент по идентификатору
    public MessageEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    // Создать элемент
    public MessageEntity create(MessageEntity entity) {
        return repository.create(entity);
    }

    // Изменить элемент
    public MessageEntity update(Long id, MessageEntity entity) {
        final MessageEntity existsEntity = get(id);
        existsEntity.setSender(entity.getSender());
        existsEntity.setText(entity.getText());
        return repository.update(existsEntity);
    }

    // Удалить элемент
    public MessageEntity delete(Long id) {
        final MessageEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
