package com.example.demo.types.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.core.configuration.Constants;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;

import jakarta.validation.Valid;

// Контроллер для сущности "Тип" (Тип книги)
@RestController
@RequestMapping(Constants.API_URL + "/type")
public class TypeController {
    // Бизнес-логика для сущности "Тип" (Тип книги)
    private final TypeService typeService;

    // Библиотека для преобразования сущности
    private final ModelMapper modelMapper;

    // Конструктор
    public TypeController(TypeService typeService, ModelMapper modelMapper) {
        this.typeService = typeService;
        this.modelMapper = modelMapper;
    }

    // Преобразовать из сущности в DTO
    private TypeDto toDto(TypeEntity entity) {
        return modelMapper.map(entity, TypeDto.class);
    }

    // Преобразовать из DTO в сущность
    private TypeEntity toEntity(@Valid TypeDto dto) {
        return modelMapper.map(dto, TypeEntity.class);
    }

    // Получить все элементы
    @GetMapping
    public List<TypeDto> getAll() {
        return typeService.getAll().stream().map(this::toDto).toList();
    }

    // Получить элемент по идентификатору
    @GetMapping("/{id}")
    public TypeDto get(@PathVariable(name = "id") Long id) {
        return toDto(typeService.get(id));
    }

    // Создать элемент
    @PostMapping
    public TypeDto create(@RequestBody @Valid TypeDto dto) {
        return toDto(typeService.create(toEntity(dto)));
    }

    // Изменить элемент
    @PutMapping("/{id}")
    public TypeDto update(@PathVariable(name = "id") Long id, @RequestBody @Valid TypeDto dto) {
        return toDto(typeService.update(id, toEntity(dto)));
    }

    // Удалить элемент
    @DeleteMapping("/{id}")
    public TypeDto delete(@PathVariable(name = "id") Long id) {
        return toDto(typeService.delete(id));
    }
}
