package com.example.demo.types.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.repository.TypeRepository;

// Бизнес-логика для сущности "Тип" (Тип книг)
@Service
public class TypeService {
    // Хранилище элементов
    private final TypeRepository repository;

    // Конструктор
    public TypeService(TypeRepository repository) {
        this.repository = repository;
    }

    // Получить все элементы
    public List<TypeEntity> getAll() {
        return repository.getAll();
    }

    // Получить элемент по идентификатору
    public TypeEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    // Создать элемент
    public TypeEntity create(TypeEntity entity) {
        return repository.create(entity);
    }

    // Изменить элемент
    public TypeEntity update(Long id, TypeEntity entity) {
        final TypeEntity existsEntity = get(id);
        existsEntity.setName(entity.getName());
        return repository.update(existsEntity);
    }

    // Удалить элемент
    public TypeEntity delete(Long id) {
        final TypeEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
