package com.example.demo.users.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.core.configuration.Constants;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

// Контроллер для сущности "Пользователь"
@RestController
@RequestMapping(Constants.API_URL + "/user")
public class UserController {
    // Бизнес-логика для сущности "Пользователь"
    private final UserService userService;

    // Библиотека для преобразования сущности
    private final ModelMapper modelMapper;

    // Конструктор
    public UserController(UserService userService, ModelMapper modelMapper) {
        this.userService = userService;
        this.modelMapper = modelMapper;
    }

    // Преобразовать из сущности в DTO
    private UserDto toDto(UserEntity entity) {
        return modelMapper.map(entity, UserDto.class);
    }

    // Преобразовать из DTO в сущность
    private UserEntity toEntity(@Valid UserDto dto) {
        return modelMapper.map(dto, UserEntity.class);
    }

    // Получить все элементы
    @GetMapping
    public List<UserDto> getAll() {
        return userService.getAll().stream().map(this::toDto).toList();
    }

    // Получить элемент по идентификатору
    @GetMapping("/{id}")
    public UserDto get(@PathVariable(name = "id") Long id) {
        return toDto(userService.get(id));
    }

    // Создать элемент
    @PostMapping
    public UserDto create(@RequestBody @Valid UserDto dto) {
        return toDto(userService.create(toEntity(dto)));
    }

    // Изменить элемент
    @PutMapping("/{id}")
    public UserDto update(@PathVariable(name = "id") Long id, @RequestBody @Valid UserDto dto) {
        return toDto(userService.update(id, toEntity(dto)));
    }

    // Удалить элемент
    @DeleteMapping("/{id}")
    public UserDto delete(@PathVariable(name = "id") Long id) {
        return toDto(userService.delete(id));
    }
}
