package com.example.demo.users.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.repository.UserRepository;

// Бизнес-логика для сущности "Пользователь"
@Service
public class UserService {
    // Хранилище элементов
    private final UserRepository repository;

    // Конструктор
    public UserService(UserRepository repository) {
        this.repository = repository;
    }

    // Получить все элементы
    public List<UserEntity> getAll() {
        return repository.getAll();
    }

    // Получить элемент по идентификатору
    public UserEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    // Создать элемент
    public UserEntity create(UserEntity entity) {
        return repository.create(entity);
    }

    // Изменить элемент
    public UserEntity update(Long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);
        existsEntity.setUsername(entity.getUsername());
        existsEntity.setPassword(entity.getPassword());
        existsEntity.setEmail(entity.getEmail());
        return repository.update(existsEntity);
    }

    // Удалить элемент
    public UserEntity delete(Long id) {
        final UserEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
