package com.example.demo;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.items.model.ItemEntity;
import com.example.demo.items.service.ItemService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
public class ItemServiceTests {
    @Autowired
    private ItemService itemService;

    @Autowired
    private TypeService typeService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> itemService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() {
        final TypeEntity type = typeService.create(new TypeEntity(null, "Protection"));
        itemService.create(new ItemEntity(null, type, 50.00, 20));
        itemService.create(new ItemEntity(null, type, 12.00, 3));
        final ItemEntity last = itemService.create(new ItemEntity(null, type, 15.00, 6));

        Assertions.assertEquals(3, itemService.getAll(0L).size());
        Assertions.assertEquals(last, itemService.get(3L));
    }

    @Test
    @Order(2)
    void updateTest() {
        final TypeEntity type = typeService.create(new TypeEntity(null, "Sharpness"));
        final ItemEntity entity = itemService.get(3L);
        final Double oldPrice = entity.getPrice();
        final ItemEntity newEntity = itemService.update(3L, new ItemEntity(null, type, 30.00, 6));

        Assertions.assertEquals(3, itemService.getAll(0L).size());
        Assertions.assertEquals(newEntity, itemService.get(3L));
        Assertions.assertEquals(30.00, newEntity.getPrice());
        Assertions.assertNotEquals(oldPrice, newEntity.getPrice());
    }

    @Test
    @Order(3)
    void deleteTest() {
        itemService.delete(3L);
        Assertions.assertEquals(2, itemService.getAll(0L).size());
        Assertions.assertThrows(NotFoundException.class, () -> itemService.get(3L));

        final TypeEntity type = typeService.create(new TypeEntity(null, "Protection"));
        final ItemEntity newEntity = itemService.create(new ItemEntity(null, type, 50.00, 10));
        Assertions.assertEquals(3, itemService.getAll(0L).size());
        Assertions.assertEquals(4L, newEntity.getId());
    }
}
