package com.example.demo;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
public class MessageServiceTests {
    @Autowired
	private MessageService messageService;

	@Autowired
	private UserService userService;

	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> messageService.get(0L));
	}

    @Test
	@Order(1)
	void createTest() {
		final UserEntity sender = userService.create(new UserEntity(null, "User1", "password1", "mail1@gmail.com"));
		messageService.create(new MessageEntity(null, sender, "Message1"));
		messageService.create(new MessageEntity(null, sender, "Message2"));
		final MessageEntity last = messageService.create(new MessageEntity(null, sender, "Message3"));
		
		Assertions.assertEquals(3, messageService.getAll(0l).size());
		Assertions.assertEquals(last, messageService.get(3L));
	}

    @Test
	@Order(2)
	void updateTest() {
		final UserEntity sender = userService.create(new UserEntity(null, "TESTuser", "password", "mail@gmail.com"));
		final MessageEntity entity = messageService.get(3L);
		final String oldText = entity.getText();
		final MessageEntity newEntity = messageService.update(3L, new MessageEntity(1L, sender, "textMessage"));
		
		Assertions.assertEquals(3, messageService.getAll(0L).size());
		Assertions.assertEquals(newEntity, messageService.get(3L));
		Assertions.assertEquals("textMessage", newEntity.getText());
		Assertions.assertNotEquals(oldText, newEntity.getText());
	}

    @Test
	@Order(3)
	void deleteTest() {
		messageService.delete(3L);
		Assertions.assertEquals(2, messageService.getAll(0L).size());
		Assertions.assertThrows(NotFoundException.class, () -> messageService.get(3L));

		final UserEntity sender = userService.create(new UserEntity(null, "User1", "password1", "mail1@gmail.com"));
		final MessageEntity newEntity = messageService.create(new MessageEntity(null, sender, "Message"));
		Assertions.assertEquals(3, messageService.getAll(0L).size());
		Assertions.assertEquals(4L, newEntity.getId());
	}
}
